@@ -2,22 +2,55 @@
 
 from __future__ import absolute_import
 
+import warnings
+
+import sys
+
 from ruamel.yaml.compat import utf8
 
-__all__ = ['Mark', 'YAMLError', 'MarkedYAMLError', 'ReusedAnchorWarning']
+if sys.version_info >= (3, 5, 2):
+    from typing import Any, Dict, Optional, List, Text  # NOQA
 
 
-class Mark(object):
-    def __init__(self, name, index, line, column, buffer, pointer):
+__all__ = [
+    'FileMark', 'StringMark', 'CommentMark', 'YAMLError', 'MarkedYAMLError',
+    'ReusedAnchorWarning', 'UnsafeLoaderWarning',
+]
+
+
+class StreamMark(object):
+    __slots__ = 'name', 'index', 'line', 'column',
+
+    def __init__(self, name, index, line, column):
+        # type: (Any, int, int, int) -> None
         self.name = name
         self.index = index
         self.line = line
         self.column = column
+
+    def __str__(self):
+        # type: () -> Any
+        where = "  in \"%s\", line %d, column %d"   \
+                % (self.name, self.line+1, self.column+1)
+        return where
+
+
+class FileMark(StreamMark):
+    __slots__ = ()
+
+
+class StringMark(StreamMark):
+    __slots__ = 'name', 'index', 'line', 'column', 'buffer', 'pointer',
+
+    def __init__(self, name, index, line, column, buffer, pointer):
+        # type: (Any, int, int, int, Any, Any) -> None
+        StreamMark.__init__(self, name, index, line, column)
         self.buffer = buffer
         self.pointer = pointer
 
     def get_snippet(self, indent=4, max_length=75):
-        if self.buffer is None:
+        # type: (int, int) -> Any
+        if self.buffer is None:  # always False
             return None
         head = ''
         start = self.pointer
@@ -38,10 +71,13 @@ class Mark(object):
                 end -= 5
                 break
         snippet = utf8(self.buffer[start:end])
+        caret = '^'
+        caret = '^ (line: {})'.format(self.line+1)
         return ' '*indent + head + snippet + tail + '\n' \
-               + ' '*(indent+self.pointer-start+len(head)) + '^'
+               + ' '*(indent+self.pointer-start+len(head)) + caret
 
     def __str__(self):
+        # type: () -> Any
         snippet = self.get_snippet()
         where = "  in \"%s\", line %d, column %d"   \
                 % (self.name, self.line+1, self.column+1)
@@ -50,6 +86,14 @@ class Mark(object):
         return where
 
 
+class CommentMark(object):
+    __slots__ = 'column',
+
+    def __init__(self, column):
+        # type: (Any) -> None
+        self.column = column
+
+
 class YAMLError(Exception):
     pass
 
@@ -57,6 +101,7 @@ class YAMLError(Exception):
 class MarkedYAMLError(YAMLError):
     def __init__(self, context=None, context_mark=None,
                  problem=None, problem_mark=None, note=None):
+        # type: (Any, Any, Any, Any, Any) -> None
         self.context = context
         self.context_mark = context_mark
         self.problem = problem
@@ -64,7 +109,8 @@ class MarkedYAMLError(YAMLError):
         self.note = note
 
     def __str__(self):
-        lines = []
+        # type: () -> Any
+        lines = []  # type: List[str]
         if self.context is not None:
             lines.append(self.context)
         if self.context_mark is not None  \
@@ -84,3 +130,18 @@ class MarkedYAMLError(YAMLError):
 
 class ReusedAnchorWarning(Warning):
     pass
+
+
+class UnsafeLoaderWarning(Warning):
+    text = """
+The default 'Loader' for 'load(stream)' without further arguments can be unsafe.
+Use 'load(stream, Loader=ruamel.yaml.Loader)' explicitly if that is OK.
+Alternatively include the following in your code:
+
+  import warnings
+  warnings.simplefilter('ignore', ruamel.yaml.error.UnsafeLoaderWarning)
+
+In most other cases you should consider using 'safe_load(stream)'"""
+    pass
+
+warnings.simplefilter('once', UnsafeLoaderWarning)
